import { SystemAttributes } from "./constants";
import { List } from "./models/list.model";
import { PageNode, PageNodeAttribute } from "./models/page-node.model";

export class ElementAttributes implements List<PageNodeAttribute> {

  get value(): PageNodeAttribute[] {
    return this.pageElement.schema.attributes || [];
  }

  private attributesMap = new Map<string, string>();

  get isNodeWithAttributes(): boolean {
    return this.pageElement.nativeElement.nodeType !== Node.TEXT_NODE;
  }

  private get hostElement(): Element {
    return this.pageElement.hostElement as Element;
  }

  constructor(
    private pageElement: PageNode,
  ) {
  }

  add(...attributes: PageNodeAttribute[]): void {
    this.setAttributes(attributes);
    this.updateSchemaAttributes();
    this.pageElement.emit('view:change');
  }

  remove(...names: string[]): void {
    this.removeAttributes(names);
    this.updateSchemaAttributes();
    this.pageElement.emit('view:change');
  }

  /**
   * @NotImplemented
   * @param name 
   * @param force 
   */
  toggle(name: string, force?: boolean): void {
    if (this.isNodeWithAttributes) {
      this.toggleAttribute(name, force);
    }

    this.pageElement.emit('view:change');
  }

  replace(name: string, attribute: PageNodeAttribute): void {
    this.removeAttributes([name]);
    this.setAttributes([attribute]);
    this.updateSchemaAttributes();
    this.pageElement.emit('view:change');
  }

  init(): void {
    this.setAttributes(this.pageElement.schema.attributes);
  }

  private updateSchemaAttributes(): void {
    this.pageElement.schema.attributes = Array.from(this.attributesMap).map(([name, value]) => ({ name, value }));
  }

  private setAttributes(attrs: PageNodeAttribute[]): void {
    attrs.forEach(attr => {
      if (this.isNodeWithAttributes) {
        this.hostElement.setAttribute(attr.name, attr.value);
      }

      if (Object.values(SystemAttributes).indexOf(attr.name as any) < 0) {
        this.attributesMap.set(attr.name, attr.value);
      }
    });
  }

  private removeAttributes(names: string[]): void {
    names.forEach(name => {
      if (this.isNodeWithAttributes) {
        this.hostElement.removeAttribute(name);
      }

      this.attributesMap.delete(name);
    });
  }

  private toggleAttribute(name: string, force?: boolean): void {
    this.hostElement.toggleAttribute(name, force);
  }
}