import { Page } from "../page";
import { IPageBaseNode, NodeType } from "../models/page-node.model";
import { v4 as uuidv4 } from 'uuid';
import { EventEmitter } from "../event-emitter";
import { ElementAttributes } from "../element-attributes";
import { IPageElementNode } from "../models/page-node.model";
import { PageNodeSchema } from "../models/element-schema.model";

export abstract class PageBaseNode<T extends PageBaseNode<T>> extends EventEmitter implements IPageBaseNode {
  key!: string;
  children?: IPageBaseNode[];

  attributes!: ElementAttributes;
  abstract nodeType: NodeType;
  abstract nativeElement: Node | Element;
  abstract schema: PageNodeSchema;
  abstract page: Page;
  abstract parentElement?: IPageElementNode | undefined;
  abstract get hostElement(): Node | Element;

  constructor() {
    super();
  }

  createNode(): void {
    this.key = uuidv4();
    this.schema.key = this.key;
  }

  findElement(predicate: (el: IPageBaseNode) => boolean): IPageBaseNode | undefined {
    return predicate(this) ? this : undefined;
  };

  destroy(): void {
    this.parentElement?.removeChild(this);
  }

  getSchema(): PageNodeSchema {
    return this.schema;
  }

  removeChild(child: IPageBaseNode): void {
    this.children = this.children?.filter(el => el !== child);
    this.nativeElement.removeChild(child.hostElement || child.nativeElement);
    this.page.schemaChanged();
  }

  changeElementIndex(newIndex: number): void {
    if (!this.parentElement) {
      return;
    }
    
    if (newIndex < 0 || newIndex > Number(this.parentElement?.children?.length)) {
      console.error('Invalid indices provided');
      return;
    }

    const parent = this.parentElement;
    const currentIndex = parent.children!.findIndex(el => el === this);
    const targetPageNode = parent.children![newIndex];
    const targetElement = targetPageNode.hostElement as Element;
    const currentElement = this.hostElement;
    
    parent.nativeElement.removeChild(currentElement);
    if (currentIndex > newIndex) {
      parent.nativeElement.insertBefore(currentElement, targetElement);
    } else {
      const nextSibling = targetElement.nextElementSibling;
      if (!nextSibling) {
        parent.nativeElement.appendChild(currentElement);
      } else {
        parent.nativeElement.insertBefore(currentElement, nextSibling);
      }
    }

    parent.children = parent.children!.filter(it => it !== this);
    parent.children.splice(newIndex, 0, this);
    this.page.schemaChanged();
  }
}