import { IPageElementNode, NodeType, PageNode } from "../models/page-node.model";
import { PageBaseNode } from "./page-base-node";
import { Page } from "../page";
import { parseHTMLFragment } from "../utils";
import { IPage } from "../models/page.model";
import { nodeFactory } from "./node-factory";
import { SystemAttributes } from "../constants";
import { ElementClassList } from "../element-classlist";
import { ElementAttributes } from "../element-attributes";
import { ElementStyles } from "../element-styles";
import { PageElementNodeSchema, PageNodeSchema } from "../models/element-schema.model";

export class PageElementNode extends PageBaseNode<PageElementNode> implements IPageElementNode {
  override nativeElement!: HTMLElement;
  override nodeType: NodeType = NodeType.element;
  override children: PageNode[] = [];

  get childElements(): PageElementNode[] {
    return this.children.filter(ch => ch instanceof PageElementNode) as PageElementNode[];
  }

  page!: Page;
  override schema!: PageElementNodeSchema;
  override hostElement!: HTMLElement;
  parentElement!: PageElementNode;

  classList!: ElementClassList;
  styles!: ElementStyles;

  constructor(
    schema: PageNodeSchema,
    page: IPage,
    parentElement?: IPageElementNode | undefined,
  ) {
    super();
    this.page = page as Page;
    this.parentElement = parentElement as PageElementNode;
    this.schema = schema as PageElementNodeSchema;
    this.classList = new ElementClassList(this);
    this.styles = new ElementStyles(this);
    this.attributes = new ElementAttributes(this);
    this.createElementNode();
  }

  private createElementNode(): void {
    super.createNode();
    const elRef = this.page.documentRef.createElement(this.schema.tag);
    this.nativeElement = elRef!;
    this.hostElement = this.nativeElement;
    this.parentElement?.nativeElement?.appendChild(this.nativeElement);


    if (this.schema.children?.length) {
      this.children = this.schema.children.map(childSchema => this.insertChild(childSchema, false));
    }

    this.attributes.add({name: SystemAttributes.nodeKey, value: this.key});
    this.nativeElement.setAttribute(SystemAttributes.nodeKey, this.key);

    this.classList.init();
    this.styles.init();
    this.attributes.init();

    this.nativeElement.tabIndex = 1;
    this.page.nodeCreated(this);
  }

  override getSchema(): PageNodeSchema {
    this.schema.children = this.children?.map(ch => ch.getSchema());
    return this.schema;
  }

  override findElement(predicate: (el: PageNode) => boolean): PageNode | undefined {
    const thisSuits = super.findElement(predicate);
    if (thisSuits) {
      return this;
    }

    const found = this.children?.map(ch =>  ch.findElement(predicate)).filter(Boolean)[0];
    if (found) {
      return found;
    }

    return undefined;
  }

  override destroy(): void {
    super.destroy();
    this.page.emit('node:destroy', this);
    this.emit('destroy');
  }

  insertChild(schema: PageNodeSchema, emitSchemaChanged = true): PageNode {
    const node = nodeFactory(schema, this.page, this);
    this.children = [...(this.children || []), node];
    this.schema.children = [...(this.schema.children || []), node.schema];
    if (emitSchemaChanged) {
      this.page.schemaChanged();
    }

    return node;
  }

  insertHTMLFragment(fragment: string): PageNode[] {
    const schemas = parseHTMLFragment(fragment);
    const children = schemas.map(schema => this.insertChild(schema, false));
    this.page.schemaChanged();
    return children;
  }
}