
import { EventEmitter } from "./event-emitter";
import { PageNodeSchema } from "./models/element-schema.model";
import { IPageElementNode, PageNode } from "./models/page-node.model";
import { PageOptions } from "./models/page-options.model";
import { IPage, IPageLibrary } from "./models/page.model";
import { nodeFactory } from "./nodes/node-factory";
import { PageCustomNode } from "./nodes/page-custom-node";
import { PageElementNode } from "./nodes/page-element-node";

export class Page extends EventEmitter implements IPage {
  private styleSheet!: CSSStyleSheet;
  private rootElement!: PageElementNode;
  private libraries: HTMLLinkElement[] = [];
  private iframe!: HTMLIFrameElement;

  get documentRef(): Document {
    return this.iframe.contentDocument || this.iframe.contentWindow?.document!;
  }

  get windowRef(): Window {
    return this.iframe.contentWindow!;
  }

  constructor(
    public container: HTMLElement,
    public title: string,
    private schema: PageNodeSchema,
    private options: PageOptions,
    public rootDocumentRef: Document
  ) {
    super();
    this.registerIframe();
    
    this.iframe.addEventListener('load', () => {
      this.registerStyleSheet();
     
      this.emit('ready');
      this.linkLibrary({ name: 'custom-styles', href: './lib-styles.css', rel: 'stylesheet' });
    });
  }

  getRootElement(): IPageElementNode {
    return this.rootElement;
  }

  getStyleSheet(): CSSStyleSheet {
    return this.styleSheet;
  }

  getLinkedLibraries(): IPageLibrary[] {
    return this.libraries.map(lib => ({
      ...Array.from(lib.attributes).reduce((r, attr) => (r[attr.name] = attr.value) && r, {} as any),
      name: lib.id
    }));
  }

  getSchema(): PageNodeSchema {
    return this.rootElement.getSchema();
  }

  schemaChanged(): void {
    this.emit('schema:change', this.getSchema());
  }

  getOptions(): PageOptions {
    return this.options;
  }

  buildElements(container: HTMLElement): void {
    this.rootElement = nodeFactory(this.schema, this) as PageElementNode;
    container.appendChild(this.rootElement.nativeElement);
  }

  customElementFactory(element: PageCustomNode): HTMLElement {
    if (!this.options.customElementFactory) {
      console.error('Custom element factory was not provided');
      return this.getStubElement();
    }

    return this.options.customElementFactory(element);
  }

  nodeCreated(node: PageNode): void {
    this.emit('node:create', node);
  }

  findElement(predicate: (el: PageNode) => boolean): PageNode | undefined {
    return this.rootElement.findElement(predicate);
  }

  linkLibrary({name, href, rel, ...rest}: {name: string; href: string; rel: string; [prop: string]: string}): Promise<void> {
    const link = this.documentRef.createElement('link');
    link.href = href;
    link.rel = rel;
    link.id = name;
    Object.entries(rest).forEach(([key, value]) => {
      link.setAttribute(key, value);
    });

    this.documentRef!.head.appendChild(link);
    return new Promise((res, rej) => {
      link.onload = () => {
        this.libraries.push(link);
        res();
      }
      link.onerror = () => {
        console.log('error linking lib');
        rej();
      }
    });
  }

  logError(node: PageNode, errorMessage: string): void {
    this.emit('node:error', { node, errorMessage });
    console.error(errorMessage);
  }

  private registerIframe(): void {
    this.iframe = this.rootDocumentRef.createElement('iframe');
    this.iframe.classList.add('page-iframe');
    this.container.appendChild(this.iframe);
  }

  private registerStyleSheet(): void {
      const styleSheet = this.documentRef.createElement('style');
      styleSheet.setAttribute('type', 'text/css');
      this.documentRef.head.appendChild(styleSheet);
      this.styleSheet = styleSheet.sheet!;
  }

  private getStubElement(): HTMLElement {
    return this.documentRef.createElement('span');
  }
}