
import { StyleType } from "../models/style.model";
import { ElementEdges } from "../models/element-edges.model";
import { PageElementNode } from "../nodes/page-element-node";
import { StyleBase } from "./style-base";
import { camelToKebabCase } from "../utils";

export type BorderStyleType = 'solid' | 'dotted';
export interface BorderRadiusEdges {
  topLeft: string;
  topRight: string;
  bottomLeft: string;
  bottomRight: string;
}

export interface IBorderStyle {
  width?: ElementEdges<string> | string;
  styleType?: ElementEdges<BorderStyleType>;
  color?: ElementEdges<string> | string;
  radius?: BorderRadiusEdges | string;
}

export class BorderStyle extends StyleBase<IBorderStyle> {
  override styleType: StyleType = StyleType.border;
  override selector!: string;
  override styleSheet!: CSSStyleSheet;

  constructor(public element: PageElementNode) {
    super();
    this.selector = `${this.styleType}-${this.element.key}`;
    this.styleSheet = this.element.page.getStyleSheet();
  }

  setWidth(width: ElementEdges<string> | string, apply = true): void {
    this.setInitialValuesOnChange('width');

    if (typeof width !== 'string') {
      this.rule.setProp('border-width', '0px');
      this.rule.setPropEdges('border-width', width, {
        getEdgePropKey: (edgeKey) => `border-${edgeKey}-width`
      });
    } else {
      this.rule.setProp('border-width', width);
      this.rule.deletePropEdges('border-width', {
        getEdgePropKey: (edgeKey) => `border-${edgeKey}-width`
      })
    }

    this.tryToApplyStyleRule(apply);
  }

  getWidth(): ElementEdges<string> | string | undefined {
    return this.rule.getPropEdges('border-width', {
      getEdgePropKey: (edgeKey) => `border-${edgeKey}-width`
    }) ?? this.rule.getProp('border-width');
  }

  setStyleType(styleType: ElementEdges<BorderStyleType> | BorderStyleType, apply = true): void {
    this.setInitialValuesOnChange('styleType');

    this.rule.setPropOrPropEdges('border-style', styleType, {
      getEdgePropKey: (edgeKey) => `border-${edgeKey}-style`
    });
    this.tryToApplyStyleRule(apply);
  }

  getStyleType(): ElementEdges<BorderStyleType> | BorderStyleType | undefined {
    return this.rule.getPropOrPropEdges<ElementEdges<BorderStyleType>, BorderStyleType>('border-style');
  }

  setColor(color: ElementEdges<string> | string, apply = true): void {
    this.setInitialValuesOnChange('color');

    this.rule.setPropOrPropEdges('border-color', color, {
      getEdgePropKey: (edgeKey) => `border-${edgeKey}-color`
    });

    this.tryToApplyStyleRule(apply);
  }

  getColor(): ElementEdges<string> | string | undefined {
    return this.rule.getPropOrPropEdges('border-color', {
      getEdgePropKey: (edgeKey) => `border-${edgeKey}-color`,
    });
  }

  setBorderRadius(radius: BorderRadiusEdges | string, apply = true): void {
    this.setInitialValuesOnChange('border-radius');

    this.rule.setPropOrPropEdges<BorderRadiusEdges>('border-radius', radius, {
      edgeKeys: ['topRight', 'topLeft', 'bottomRight', 'bottomLeft'],
      getEdgePropKey: (edgeKey) => `border-${camelToKebabCase(edgeKey)}-radius`
    });

    this.tryToApplyStyleRule(apply);
  }

  getBorderRadius(): BorderRadiusEdges | string | undefined {
    return this.rule.getPropOrPropEdges<BorderRadiusEdges>('border-radius', {
      edgeKeys: ['topRight', 'topLeft', 'bottomRight', 'bottomLeft'],
      getEdgePropKey: (edgeKey) => `border-${camelToKebabCase(edgeKey)}-radius`
    });
  }

  private setInitialValuesOnChange(changedProp: 'width' | 'styleType' | 'color' | 'border-radius'): void {
    if (changedProp !== 'width' && this.getWidth() === undefined) {
      this.rule.setProp('border-width', '1px');
    }

    if (changedProp !== 'styleType' && this.getStyleType() === undefined) {
      this.rule.setProp('border-style', 'solid');
    }

    if (changedProp !== 'color' && this.getColor() === undefined) {
      this.rule.setProp('border-color', '#000000');
    }
  }
}