import { StyleType } from "../models/style.model";
import { PageElementNode } from "../nodes/page-element-node";
import { isCssPropValueValid } from "../utils";
import { StyleBase } from './style-base';

export interface ShadowEffect {
  inner: boolean;
  x: string;
  y: string;
  blur: string;
  spread: string;
  color: string;
}

export type Effect = ShadowEffect;

export class BoxShadowStyle extends StyleBase {
  override styleType: StyleType = StyleType.boxShadow;
  override styleSheet: CSSStyleSheet;
  override selector: string;

  constructor(public element: PageElementNode) {
    super();
    this.selector = `${this.styleType}-${this.element.key}`;
    this.styleSheet = this.element.page.getStyleSheet();
  }

  has(effect: ShadowEffect): boolean {
    const str = this.effectToString(effect);
    return !!this.rule.getProp<string>('box-shadow')?.includes(str);
  }

  set(effect: ShadowEffect, apply = true): void {
    const shadowProp = this.effectToString(effect);
    const boxShadowValue = this.rule.getProp<string>('box-shadow') ?? '';
    const updatedBoxShadowValue =  `${boxShadowValue}${boxShadowValue?.length ? ', ' : ''}${shadowProp}`;
    this.rule.setProp('box-shadow', updatedBoxShadowValue);

    this.tryToApplyStyleRule(apply);
  }

  getEffects(): ShadowEffect[] {
    return this.rule.getProp<string>('box-shadow')?.split(', ').map((prop: string) => this.stringToEffect(prop)) || [];
  }

  delete(effect: ShadowEffect, apply = true): void {
    if (!this.rule.getProp<string>('box-shadow')?.length) {
      return;
    }

    const shadowProp = this.effectToString(effect);
    const boxShadowProps = this.rule.getProp<string>('box-shadow')!.split(', ');
    this.rule.setProp('box-shadow', boxShadowProps.filter((prop: string) => shadowProp !== prop).join(', '));

    this.tryToApplyStyleRule(apply);
  }

  isValidValue(effect: ShadowEffect): boolean {
    return isCssPropValueValid('box-shadow', this.effectToString(effect));
  }

  private effectToString(shadow: ShadowEffect): string {
    return `${shadow.inner ? 'inset ' : ''}${shadow.x} ${shadow.y} ${shadow.blur} ${shadow.spread} ${shadow.color}`;
  }

  private stringToEffect(str: string): ShadowEffect {
    const items = str.split(' ');
    const inner = items[0] === 'inset';
    const [x, y, blur, spread, color] = items.filter(item => item !== 'inset');
    return { inner, x, y, blur, spread, color };
  }
}