import { StyleType } from "../models/style.model";
import { ElementEdges } from "../models/element-edges.model";
import { PageElementNode } from "../nodes/page-element-node";
import { StyleBase } from "./style-base";

export type Position = 'start' | 'center' | 'end' | 'space-between';

export type ItemsPosition = { direction: PositionDirection; x: Position; y: Position; gap?: string };
export type PositionDirection = 'row' | 'column';

export interface ILayoutStyle {
  backgroundColor?: string;
  margin?: ElementEdges<string> | string;
  padding?: ElementEdges<string> | string;
  position?: ItemsPosition;
}

export class LayoutStyle extends StyleBase<ILayoutStyle> {
  override styleType: StyleType = StyleType.layout;
  override styleSheet: CSSStyleSheet;
  override selector: string;

  constructor(public element: PageElementNode) {
    super();
    this.selector = `${this.styleType}-${this.element.key}`;
    this.styleSheet = this.element.page.getStyleSheet();
  }

  setPosition({ direction, x, y, gap }: ItemsPosition, apply = true): void {
    this.rule.setProp('display', 'flex');
    this.rule.setProp('flex-direction', direction);
    this.rule.setProp('gap', gap ?? '');

    if (direction === 'row') {
      this.rule.setProp('justify-content', x === 'space-between' || x === 'center' ? x : `flex-${x}`);
      this.rule.setProp('align-items', y === 'center' ? y : `flex-${y}`);
    } else if (direction === 'column') {
      this.rule.setProp('justify-content', y === 'center' || y === 'space-between' ? y : `flex-${y}`);
      this.rule.setProp('align-items', x === 'center' ? x : `flex-${x}`);
    }

    this.tryToApplyStyleRule(apply);
  }

  getPosition(): ItemsPosition | undefined {
    if (this.rule.getProp('display') !== 'flex') {
      return undefined;
    }

    const p: ItemsPosition = {} as any;
    p.direction = this.rule.getProp('flex-direction') as PositionDirection;
    let xRule, yRule;
    if (p.direction === 'row') {
      xRule = this.rule.getProp<string>('justify-content')! as string;
      yRule = this.rule.getProp<string>('align-items')! as string;
    } else {
      xRule = this.rule.getProp<string>('align-items')! as string;
      yRule = this.rule.getProp<string>('justify-content')! as string;
    }
    
    p.x = xRule === 'space-between' || xRule === 'center' ? xRule : xRule.split('-')[1] as Position;
    p.y = yRule === 'space-between' || yRule === 'center' ? yRule : yRule.split('-')[1] as Position;
    p.gap = this.rule.getProp('gap');
    return p;
  }

  deletePosition(apply = true): void {
    this.rule.deleteProp('display');
    this.rule.deleteProp('flex-direction');
    this.rule.deleteProp('justify-content');
    this.rule.deleteProp('align-items');
    this.rule.deleteProp('gap');

    this.tryToApplyStyleRule(apply);
  }

  setPadding(padding: ElementEdges<string> | string, apply = true): void {
    this.rule.setPropOrPropEdges('padding', padding);

    this.tryToApplyStyleRule(apply);
  }

  getPadding(): ElementEdges<string> | string | undefined {
    return this.rule.getPropOrPropEdges('padding');
  }

  deletePadding(apply = true): void {
    this.rule.deleteProp('padding');
    this.rule.deletePropEdges('padding');

    this.tryToApplyStyleRule(apply);
  }

  setMargin(margin: ElementEdges<string> | string, apply = true): void {
    this.rule.setPropOrPropEdges('margin', margin);

    this.tryToApplyStyleRule(apply);
  }

  getMargin(): ElementEdges<string> | string | undefined {
    return this.rule.getPropOrPropEdges('margin');
  }

  deleteMargin(apply = true): void {
    this.rule.deleteProp('margin');
    this.tryToApplyStyleRule(apply);
  }

  setBackgroundColor(color: string, apply = true): void {
    this.rule.setProp('background-color', color);

    this.tryToApplyStyleRule(apply);
  }

  getBackgroundColor(): string | undefined {
    return this.rule.getProp('background-color');
  }

  deleteBackgroundColor(apply = true): void {
    this.rule.deleteProp('background-color');
    this.tryToApplyStyleRule(apply);
  }
}