import { StyleType } from "../models/style.model";
import { PageElementNode } from "../nodes/page-element-node";
import { StyleRule } from "./style-rule";

export abstract class StyleBase<T = Record<string, string>> {
  abstract styleSheet: CSSStyleSheet;
  abstract styleType: StyleType;
  abstract selector: string;
  abstract element: PageElementNode;
  protected rule = new StyleRule();
  protected ruleText?: string;
  protected ruleIndex = -1;

  setStyleRule(ruleValue: T): void {
    this.rule.init(ruleValue as Record<string, string>);
    this.applyStyleRule();
  }

  applyStyles(): void {
    this.applyStyleRule();
    this.element.page.schemaChanged();
  }

  protected tryToApplyStyleRule(apply = true): void {
    if (apply) {
      this.applyStyles();
    }
  }

  private applyStyleRule(): void {
    const ruleText = this.rule.generateRuleText();
    if (!ruleText?.length && this.ruleIndex >= 0) {
      this.hardDeleteRule();
      this.updateNodeSchema();
      this.element.emit('view:change');
      return;
    }

    if (ruleText === this.ruleText) {
      return;
    }

    this.ruleText = ruleText;
    if (this.ruleIndex! >= 0) {
      this.updateRule();
    } else {
      this.insertRule();
    }

    
    if (!this.element.nativeElement.classList.contains(this.selector)) {
      this.element.nativeElement.classList.add(this.selector);
    }
    
    this.updateNodeSchema();
    this.element.emit('view:change');
  }

  private insertRule(): void {
    this.ruleIndex = this.styleSheet.insertRule(`.${this.selector}{${this.ruleText}}`, this.ruleIndex >= 0 ? this.ruleIndex : this.styleSheet.cssRules.length);
  }

  private updateRule(): void {
    this.deleteRule();
    this.insertRule();
  }

  private deleteRule(): void {
    this.styleSheet.deleteRule(this.ruleIndex);
  }

  private hardDeleteRule(): void {
    if (!this.styleSheet.cssRules.length) {
      console.warn('No rules were applied, nothing to delete');
      return
    }
    this.deleteRule();
    this.ruleText = '';
    this.ruleIndex = -1;
  }

  private updateNodeSchema(): void {
    if (!this.ruleText?.length) {
      delete this.element.schema.styles?.[this.styleType];
    } else {
      this.element.schema.styles = {
        ...this.element.schema.styles,
        [this.styleType]: this.rule.getAllProps()
      };
    }
  }
}