import { ElementEdges } from "../models/element-edges.model";

interface PropEdgesOptions<T> {
  edgeKeys?: (keyof T)[],
  getEdgePropKey: (edgeKey: keyof T) => string
}

export class StyleRule {
  private rule: Record<string, unknown> = {};
  private detachedProps = new Set<string>();

  getAllProps(): Record<string, unknown> {
    return Object.keys(this.rule).filter(key => !!this.rule[key]).reduce((r, key) => {
      r[key] = this.rule[key];
      return r;
    }, {} as Record<string, unknown>);
  }

  setProp<T = string>(key: string, value: T): void {
    this.rule[key] = value;
  }

  getProp<T = string>(key: string): T | undefined {
    return this.rule[key] as T;
  }

  getPropOrPropEdges<T = ElementEdges<string>, R = string>(key: string, options?: PropEdgesOptions<T>): T | R | undefined {
    return this.rule[key] as R ?? this.getPropEdges<T>(key, options);
  }

  getPropEdges<T = ElementEdges<string>>(key: string, options?: PropEdgesOptions<T>): T | undefined {
    const edges = options?.edgeKeys ?? ['left', 'top', 'right', 'bottom'] as (keyof T)[];
    const edgePropKeyFactory = options?.getEdgePropKey ?? ((edgeKey: keyof T) => `${key}-${edgeKey as string}`);
    const edgesRule = edges.reduce((r, edge) => {
      const edgePropKey = edgePropKeyFactory(edge);
      if (this.rule[edgePropKey]) {
        r[edge as keyof T] = this.rule[edgePropKey] as T[keyof T];
      }

      return r;
    }, {} as T);

    return Object.keys(edgesRule as object).length ? edgesRule : undefined;
  }

  setPropOrPropEdges<T = ElementEdges<string>, R = string>(key: string, value: T | R, options?: PropEdgesOptions<T>): void {
    if (typeof value === 'string') {
      this.deletePropEdges(key, options);
      this.rule[key] = value;
      return;
    }

    this.deleteProp(key);
    this.setPropEdges(key, value as T, options);
  }

  setPropEdges<T = ElementEdges<string>>(key: string, value: T, options?: PropEdgesOptions<T>): void {
    const edgePropKeyFactory = options?.getEdgePropKey ?? ((edgeKey: keyof T) => `${key}-${edgeKey as string}`);

    Object.entries(value as object).forEach(([edgeKey, value]) => {
      const edgePropKey = edgePropKeyFactory(edgeKey as keyof T);
      this.rule[edgePropKey] = value;
    });
  }

  attachProp(key: string): void {
    this.detachedProps.delete(key);
  }

  detachProp(key: string): void {
    this.detachedProps.add(key);
  }

  detached(key: string): boolean {
    return this.detachedProps.has(key);
  }

  init(rule: Record<string, string>): void {
    this.rule = rule;
  }

  deleteProp(key: string): void {
    delete this.rule[key];
  }

  deletePropEdges<T>(key: string, options?: PropEdgesOptions<T>): void {
    const edges = options?.edgeKeys ?? ['left', 'top', 'right', 'bottom'] as (keyof T)[];
    const edgePropKeyFactory = options?.getEdgePropKey ?? ((edgeKey: keyof T) => `${key}-${edgeKey as string}`);
    edges.forEach(edgeKey => {
      const edgePropKey = edgePropKeyFactory(edgeKey);
      delete this.rule[edgePropKey];
    });
  }

  entries(): [string, unknown][] {
    return Object.entries(this.rule);
  }

  generateRuleText(): string {
    return this.entries()
      .filter(([key, value]) => !this.detachedProps.has(key) && !!value)
      .reduce((result, [key, value]) => {
        const ruleText = `${key}: ${value};`;
        result += ruleText;
        return result;
      }, '')
  }


}